# Package: PENTATrainer2
# Version: 1.2
# Filename: import.praat
#
# Author: Santitham Prom-on, Yi Xu
# Last updated: 4 July 2016
#
# Objective: convert user-defined TextGrid into the annotate_short format

directory$ = chooseDirectory$ ("Choose a directory to annotate")
if directory$ = ""
	#[EXIT_00] Choose to not Continue
	exit
endif

#Check validity of the directory
Create Strings as file list... FileList 'directory$'/*.wav
numberOfFiles = Get number of strings
if !numberOfFiles
	select Strings FileList
	Remove
	Create Strings as file list... FileList 'directory$'/*.WAV
	numberOfFiles = Get number of strings
endif

if numberOfFiles = 0
	#[EXIT_A01] No sound files in the directory	
	exit Error Code A01: No sound files in the directory
else
	Write to raw text file... 'directory$'/FileList.txt
endif

select Strings FileList
fileName$ = Get string... 1
name$ = fileName$ - ".wav" - ".WAV"
Create Strings as file list... extension 'directory$'/'name$'.*
nExtensions = Get number of strings

i=1
while i<= nExtensions 
	select Strings extension
	ext$ = Get string... i
	nowarn Read from file... 'directory$'/'ext$'
	fullName$ = selected$()
	type$ = extractWord$(fullName$, "")
	Remove
	if type$ != "TextGrid"
		select Strings extension
		Remove string... i
		i = i - 1
		nExtensions = nExtensions -1
	endif
	i = i + 1
endwhile

#Prompt the user to select the TextGrid file extension
beginPause("Import Annotation - File Extension")
	comment("Select file extension to import")
	optionMenu("File extension:", 1)
	for i from 1 to nExtensions
		select Strings extension
		ext$ = Get string... i
		pos = length(ext$) - rindex(ext$,".")
		ext$ = right$(ext$,pos)
		option(".'ext$'")
	endfor
clicked = endPause("Cancel","Continue",2,1)

#[EXIT_00] Choose to not Continue
if clicked = 1
	exit
endif

#Extract the file extension
select Strings extension
ext$ = Get string... file_extension
pos = length(ext$) - rindex(ext$,".")
ext$ = right$(ext$,pos)

#Prompt the user to select the tier name
select Strings FileList
fileName$ = Get string... 1
name$ = fileName$ - ".wav" - ".WAV"
Read from file... 'directory$'/'name$'.'ext$'
ntier = Get number of tiers
beginPause("Import Annotation - Tier")
	comment("Mark the tier names to import")
	for i from 1 to ntier
		select TextGrid 'name$'
		tier_name$ = Get tier name... i
		boolean("Tier'i' ('tier_name$')",1)
	endfor
clicked = endPause("Cancel","Start",2,1)
select TextGrid 'name$'
Remove

for i from 1 to numberOfFiles
	#Duplicate and remove the unselected tiers, if any
	select Strings FileList
	fileName$ = Get string... i
	name$ = fileName$ - ".wav" - ".WAV"
	Read from file... 'directory$'/'name$'.'ext$'
	select TextGrid 'name$'
	ntier = Get number of tiers
	start_time = Get start time
	end_time = Get end time
	Copy... 'name$'_imported
	
	for j from 1 to ntier
		k = ntier + 1 - j
		if tier'k' = 0
			select TextGrid 'name$'_imported
			Remove tier... 'k'
		endif
	endfor

	select TextGrid 'name$'_imported
	new_ntier = Get number of tiers

	left_edge = end_time
	#Synchronize the left edges
	for j from 1 to new_ntier
		t = Get end point... j 1
		if t < left_edge
			left_edge = t
		endif
	endfor
	
	for j from 1 to new_ntier
		label$ = Get label of interval... j 2
		Remove right boundary... j 1
		Insert boundary... j left_edge
		Set interval text... j 1 
		Set interval text... j 2 'label$'
	endfor
	
	right_edge = start_time
	#Synchronize the right edges
	for j from 1 to new_ntier
		k = Get number of intervals... j
		t = Get start point... j k
		if t > right_edge
			right_edge = t
		endif
	endfor
	
	for j from 1 to new_ntier
		k = Get number of intervals... j
		Remove left boundary... j k
		Insert boundary... j right_edge
	endfor
	
	printline 'name$' 'left_edge' 'right_edge'

	select TextGrid 'name$'_imported
	Write to short text file... 'directory$'/'name$'.annotation_short

	select TextGrid 'name$'
	plus TextGrid 'name$'_imported
	Remove
endfor

printline Your annotation files have been imported.
printline Run PENTAtrainer2's Annotation tool to complete the annotation.

select Strings FileList
plus Strings extension
Remove